/*
 * Autor: Piotr Miecielica J1ISI
 * Temat: Biblioteka STL opis algorytmw : remove_copy, remove_copy_if
 *
 * Aby korzysta z algorytmw STL naley zaczy(#include) bibliotek algorithm.
 *
 *
 * Deklaracja funkcji replace_copy:
 *
 * template < class InputIterator, class OutputIterator, class T >
 * 	OutputIterator replace_copy ( 	InputIterator first, InputIterator last,
 * 									OutputIterator result, const T& old_value,
 * 									const T& new_value );
 *
 *
 * Deklaracja funkcji replace_copy_if:
 *
 * template < class InputIterator, class OutputIterator, class Predicate, class T >
 * 	OutputIterator replace_copy_if ( 	InputIterator first, InputIterator last,
 * 										OutputIterator result, Predicate pred,
 * 										const T& new_value );
 *
 *
 * Dziaanie algorytmw replace_copy i replace_copy_if polega na kopiowaniu wartoci
 * kontenera (zakres okrelony przez first i last) do miejsca, ktrego pocztek wskazuje iterator result.
 * Jeli aktualnie kopiowany element ma warto rwn old_value (w przypadku replace_copy) bd
 * speniajc predykat pred (w przypadku replace_copy_if), to w zamian jest kopiowana warto
 * wyznaczona przez new_value.
 *
 * Wynik zwracany: iterator wskazujcy na element za ostatnim skopiowanym elementem zbioru do ktrego
 * kopiowalimy
 *
 *
 * Warto zauway, e algorytm moe zmodyfikowa zbir ktry przeglda(wszystko zaley od tego na co
 * wskazuje result). Kontener do ktrego kopiujemy musi mie zakres sigajcy co najmniej
 * result + (last-first).
 */

#include<iostream>
#include<algorithm>
#include<functional> //dla unary_function
#include<list>
#include<vector>

using namespace std;



//Pomocnicza funkcja szablonowa do wyswietlania elementw z zakresu [beg,end)

template <class InputIterator>
	void print(const InputIterator& beg, const InputIterator& end)
{
	if(beg == end) {
 		 cout << "Brak elementow!" << endl;
 		 return;
 	}
	int i = 0;
	InputIterator it=beg;
	while(it != end) {
       cout << "[" << ++i << "]:= " << *it << endl;
	   ++it;
	}
}

/*
 * Bardzo prosty funktor - zwraca warto true, jeli porwnywane string s takie same
 */
template <typename T>
 struct MyPredicate : public unary_function<T,bool>{
	private:
		T value;

	public:
		MyPredicate(const T& v): value(v){}
		bool operator()(const T& v)const {
			if( value==v ) return true;
			return false;
		}
};

/*
 * funkcja sprawdza czy podany string ma mniej ni 14 znakw i czy koczy si na liter 'e'
 * zwraca true przy spenienieniu powyszych warunkw lub false w przeciwnym wypadku
 */
bool check(const string& v)
{
	if(v.size()<14 && *--v.end()=='e') return true;
	return false;
}



int main(int argc, char** argv){

	typedef bool (*MyFunPred)(const std::string&);


	list<string> difficult_words; //lista angielskich sw, w pisowni ktrych atwo si pomyli

	//przykadowe wypenienie listy
	difficult_words.push_back("victoriously"); difficult_words.push_back("non-flammable");
	difficult_words.push_back("acquaintance"); difficult_words.push_back("irresponsible");
	difficult_words.push_back("entertainingly"); difficult_words.push_back("enthronement");
	//celowo wprowadzone bdne nazwy:
	difficult_words.push_back("adress"); difficult_words.push_back("leav");


	//Wywietlanie zawartoci listy trudnych sw
	list<string>::const_iterator beg=difficult_words.begin();
	list<string>::const_iterator end=difficult_words.end();
	list<string>::iterator result=difficult_words.begin();
	cout<<"Poczatkowa zawartosc listy trudnych wyrazow:"<<endl;
	print(beg, end);


	/*
	 * UYCIE:
	 * Po powyszym wywietleniu wyapujemy, e lista zawiera bdnie napisane wyrazy.
	 * korekty mona dokona w bardzo prosty sposb.
	 *
	 * uyta funkcja: replace_copy
	 */

	cout<<endl<<"W wyniku: replace_copy(beg, end, result, string(\"leav\"), string(\"leaf\") )"
		<<" zawartosc listy trudnych wyrazow:"<<endl;
	replace_copy( beg, end, result, string("leav"), string("leaf") );
	print(beg, end); //drukujemy zawartosc listy

	/*
	 * UYCIE:
	 * Ten sam problem co wyej, rozwizany z wykorzystaniem predykatu.
	 *
	 * uyta funkcja: replace_copy_if
	 */
	cout<<endl<<"W wyniku: replace_copy_if(beg, end, result, MyPredicate<string>(\"adress\"), \"address\")"
		<<" zawartosc listy trudnych wyrazow:"<<endl;
	replace_copy_if(beg, end, result, MyPredicate<string>( string("adress") ) , string("address"));
	print(beg, end); //drukujemy zawartosc listy

	/*
	 * UYCIE:
	 *
	 * Inny przykad uycia replace_copy_if, ktry nie jest moliwy do osignicia za pomoc replace_copy
	 * Za predykat posuy funkcja check.
	 * Tym razem jednak zbir, ktry przeglda replace_copy_if nie bdzie modyfikowany - wynik dziaania
	 * zostanie zapisany do innego kontenera. Kontener ten bdzie innego typu.
	 *
	 * uyta funkcja: relace_copy_if
	 */


	vector<string> vec( difficult_words.size() ); //utworzenie nowego kontenera o odpowiednim rozmiarze
	vector<string>::iterator result_vec = vec.begin();

	cout<<endl<<"W wyniku: replace_copy_if(beg, end, result_vec, (MyFunPred)check, \"possessive vector entry\")"
		<<" zawartosc nowo utworzonego wektora trudnych wyrazow:"<<endl;
	replace_copy_if(beg, end, result_vec, (MyFunPred)check, string("possessive vector entry"));
	print(result_vec, vec.end()); //drukujemy zawartosc wektora
	return 0;
}

